/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file    events_queue.h
 *
 *  @brief   Events queue.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/

#ifndef _EVENTS_QUEUE_H_
#define _EVENTS_QUEUE_H_

#include "records.h"

typedef struct _EvQueueItem_t
{
  EventRecord_t *ev;
  struct _EvQueueItem_t  *next;
} EvQueueItem_t;

typedef struct
{
  EvQueueItem_t *first;
  EvQueueItem_t *last;
  uint32_t num_events;
} EventsQueue_t;

typedef struct
{
  EventsQueue_t *entry[ MAX_THREAD ];
} EntryStack_t;

/* EvQueueItem_t functions */
EvQueueItem_t *CreateCopyEvItem( EventRecord_t *ev );
EvQueueItem_t *CreateEvItem( EventRecord_t *ev );
void FreeEvItem( EvQueueItem_t *item );
void GetEvent( EvQueueItem_t *item, EventRecord_t **ev );

/* EventsQueue_t functions */
EventsQueue_t *CreateEventsQueue( void );
void FreeEventsQueue( EventsQueue_t *eq );
void InsertEvent( EventsQueue_t *eq, EventRecord_t *er );
void PushEvent( EventsQueue_t *eq, EventRecord_t *er, bool_t create );

bool_t IsEmptyEventsQueue( EventsQueue_t *eq );
void DequeueFirstEvent( EventsQueue_t *eq );
void GetFirstEvent( EventsQueue_t *eq, EventRecord_t **ev );
EventRecord_t *FirstEventsQueue( EventsQueue_t *eq ); /* gets first and extracts */

EntryStack_t *CreateEntryStack( void );
void PushEventStack( EntryStack_t *stack, EventRecord_t *er, uint32_t level, bool_t create );
void FreeEntryStack( EntryStack_t *stack );
bool_t IsEmptyEventsStackQueue( EntryStack_t *stack, uint32_t level );

#endif /* _EVENTS_QUEUE_H_ */
