/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file    otf_symbolic_info.h
 *
 *  @brief   Implements table to translate symbolic tokens from otf to prv.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/

#ifndef _OTF_SYMBOLIC_INFO_H_
#define _OTF_SYMBOLIC_INFO_H_

#include <inttypes.h>
#include "types.h"

#define MAX_OTF_SYMBOLIC_TOKENS 4096

#define COUNTER_GROUP_TYPE  'C'
#define COUNTER_TYPE        'c'
#define FUNCTION_GROUP_TYPE 'F'
#define FUNCTION_TYPE       'f'

typedef struct
{
  unsigned int  otf_token_id;
  char otf_token_type;
} OtfToken_Key_t;  

typedef struct
{
  unsigned int  otf_token_id; /* Negative tokens could be possible? */
  unsigned int  otf_token_group;
  int prv_color; /* for pcf */
  char *otf_token_name;
  char otf_token_type;
  unsigned int alias_id;
  unsigned int masked_id;
} OtfToken_t;

typedef struct
{
  unsigned int num_tokens;
  OtfToken_t token_info[ MAX_OTF_SYMBOLIC_TOKENS ];
} SymbolicInfo_t;

SymbolicInfo_t *sym_otf_info;


OtfToken_t *Create_Otf_Token( char         token_type,
                              unsigned int token_group,
                              unsigned int token_id,
                              const char *token_name,
                              unsigned int alias_id,
                              unsigned int masked_id );
void Print_Otf_Token( OtfToken_t *token, int ii );
void Free_Otf_Token( OtfToken_t *token );

SymbolicInfo_t *Create_Otf_Symbolic_Info( void );
void Insert_Otf_Token( OtfToken_t *token, SymbolicInfo_t *sym_info );
void Insert_Otf_Info( char         token_type,
                      unsigned int token_group,
                      unsigned int token_id,
                      const char *token_name,
                      SymbolicInfo_t *sym_info,
                      unsigned int masked_id );
unsigned int Inserted_Tokens( SymbolicInfo_t *sym_info );
bool_t Get_Otf_Token_Group( SymbolicInfo_t *sym_info, unsigned int token_id,
                                  char type, unsigned int *token_group );
unsigned int Get_Otf_Token_AliasId( SymbolicInfo_t *sym_info,
                                  unsigned int token_id,
                                  char type,
                                  unsigned int *masked_id );
unsigned int Get_Token_MaskedId( SymbolicInfo_t *sym_info,
                                   unsigned int alias_id);
void Set_Token_MaskedId( SymbolicInfo_t *sym_info,
                          unsigned int alias_id,
                          unsigned int masked_id);

void Print_Otf_Symbolic_Info( SymbolicInfo_t *sym_info );
void Free_Otf_Symbolic_Info( SymbolicInfo_t *sym_info );

void Generate_PCF( SymbolicInfo_t *sym_info, FILE *fd );

bool_t Is_MPI_Wait( SymbolicInfo_t *sym_info, unsigned int token_value);
bool_t Is_MPI_Send( SymbolicInfo_t *sym_info, unsigned int token_value);
bool_t Is_MPI_Recv( SymbolicInfo_t *sym_info, unsigned int token_value);

#endif /* _OTF_SYMBOLIC_INFO_H_ */
