/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file   paraver_header.c
 *
 *  @brief  Provides ParaverHeader_t operations.
 *
 *  @author Pedro A. Gonzalez - CEPBA-tools
 */

#ifndef _PARAVER_HEADER_H_
#define _PARAVER_HEADER_H_

/* ---[ SYSTEM INCLUDES ]--------------------------------------------------- */
#include <inttypes.h>

/* ---[ OTF2PRV INCLUDES ]--------------------------------------------------- */
#include "paraver_hierarchy.h"

/* ---[ CONSTANTS ]---------------------------------------------------------- */
#define TIME_UNIT_LABEL_LENGTH 3

/* ---[ STRUCTURES DEFINITION ]---------------------------------------------- */
typedef struct
{
  char *time_header;
  char time_units[TIME_UNIT_LABEL_LENGTH];
  uint64_t max_time;
  ProcessHierarchy_t  *process_hier;
  ResourceHierarchy_t *resource_hier;

} ParaverHeader_t;

/* ---[ HEADERS ]------------------------------------------------------------ */

/**
 * Allocates and initializes ParaverHeader_t fields.
 *
 * @param proc_hierarchy  (In) Reference to predefined ProcessHierarchy_t, or
 *                        NULL if none defined. In both cases it's copied.
 * @param res_hierarchy   (In) Reference to predefined ResourceHierarchy_t,
 *                        or NULL if none defined. In both cases it's copied.
 * @return  Initialized ParaverHeader_t structure, or NULL if error.
 */
ParaverHeader_t *Create_Paraver_Header();


void Set_Max_Time( ParaverHeader_t *header, uint64_t time );
uint64_t Get_Max_Time( ParaverHeader_t *header );

/**
 * Creates a file named 'header_file_name', containing header information
 * gathered from ParaverHeader_t fields.
 *
 * @param header_file_name  (In) Reference to predefined ProcessHierarchy_t, or
 *                          NULL if none defined.
 * @return  Header file, or NULL if error.
 */
void Generate_Paraver_Header( ParaverHeader_t *header,  FILE *fd );


/**
 * Free allocated memory in ParaverHeader_t structure.
 *
 * @param header  (In/Out) Pointer to ParaverHeader_t structure to be freed.
 */
void Free_Paraver_Header( ParaverHeader_t *header );

#endif /* _PARAVER_HEADER_H_ */
