/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file   paraver_hierarchy.h
 *
 *  @brief  Provides ProcessHierarchy_t and ResourceHierarchy_t operations.
 *
 *  @author Pedro A. Gonzalez - CEPBA-tools
 */

#ifndef _PARAVER_HIERARCHY_H_
#define _PARAVER_HIERARCHY_H_

#include <inttypes.h>

#define DEFAULT_CPUS_PER_NODE ((unsigned int)1)
#define MAX_NODES         8192
#define MAX_APPLICATIONS     1 
#define MAX_TASKS        65535
#define MAX_THREADS          1
/* #define MAX_COUNTERS      4096 */
#define MAX_COUNTERS        16


typedef struct
{
  unsigned int nthreads;
  unsigned int node;
  unsigned int *global_cpu;
  uint32_t      ncounters;
  uint64_t      *counter_value;  /* Counters per task */
}Task_t;

typedef struct
{
   unsigned int ntasks;
   Task_t *tasks;
   unsigned int id_group;
  uint64_t      *counter_index;  
  uint32_t      ncounters; /* Global counters */
}Application_t;


typedef struct
{
  unsigned int nappls;
  Application_t *appls;

} ProcessHierarchy_t;

typedef struct
{
  unsigned int ncpus;
  unsigned int global_cpu;

} NodeInfo_t;

typedef struct
{
  unsigned int nnodes;
 /*  CPUInfo_t *node_info; */
  NodeInfo_t *node_info;
  unsigned int ncpus;

} ResourceHierarchy_t;

ProcessHierarchy_t *Create_Process_Hierarchy( void );
void Print_Process_Hierarchy( ProcessHierarchy_t *res );
void Free_Process_Hierarchy( ProcessHierarchy_t *ph );

void InsertCounter( ProcessHierarchy_t *ph, uint32_t counter );
uint64_t GetCounterValue(ProcessHierarchy_t *ph, unsigned int task, uint32_t counter );
void SetCounterValue(ProcessHierarchy_t *ph, unsigned int task, uint32_t counter, uint64_t value );

ResourceHierarchy_t *Create_Resource_Hierarchy( void );
void Print_Resource_Hierarchy( ResourceHierarchy_t *res );
void Free_Resource_Hierarchy( ResourceHierarchy_t *res );

#endif /*_PARAVER_HIERARCHY_H_ */
