/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file  parse_parameters.h
 *
 *  @brief  Translates traces between otf and prv format.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/
#ifndef _PARSE_PARAMETERS_H_
#define _PARSE_PARAMETERS_H_

#include "tokens.h"
#include "types.h"

#define USAGE_MSG "\nUsage:\n" \
  "  otf2prv [OPTIONS] trace.otf [trace.prv]\n\n"

#define OPTIONS_MSG "Options:\n"\
  "  -h\n"\
  "  --help\n"\
  "        Display this information.\n"\
  "  -l\n"\
  "  --log\n"\
  "        Translation information is logged on a generated \"trace.log\" file.\n"\
  "  -v\n"\
  "  --verbose\n"\
  "        Verbose mode ON. Default is OFF.\n"\
  "  -i2\n"\
  "  --ignore-events\n"\
  "        Doesn't translate Paraver event records (type 2).\n"\
  "  -i3\n"\
  "  --ignore-comms\n"\
  "        Doesn't translate Paraver communication records (type 3).\n\n"

#define PARAMETERS_MSG "Parameters:\n"\
  "  trace.otf\n"\
  "        Name of the input otf file. Preffix \"trace\" will be used as\n"\
  "        global name.\n"\
  "  trace.prv\n"\
  "        Name of the output Paraver trace.\n\n"


/* Options index: tokens table extended with following OPT defines. */
#define OPTION_PRV_GZIPPED (OPTION_ZIP+1)
#define OPTION_OTF_GZIPPED (OPTION_ZIP+2)

#define NUM_OPTIONS OPTION_OTF_GZIPPED

#define SUFFIX_PRV        "prv"
#define SUFFIX_DOT_PRV    ".prv"
#define SUFFIX_DOT_PRV_GZ ".prv.gz"

#define SUFFIX_DOT_OTF    ".otf"

#define SUFFIX_PCF "pcf"
#define SUFFIX_EVT "evt"
#define SUFFIX_COM "cmm"
#define SUFFIX_ERR "err"

typedef struct
{
  char *prv_path;
  char *prv_trace_name;
  char *otf_path;
  char *otf_trace_name;
  char *events_path;
  char *events_trace_name;
  bool_t option[NUM_OPTIONS+1];

} TranslatorOptions_t;

void Parse_Options( int argc , char *argv[], TranslatorOptions_t *param );
void Show_Usage( void );

TranslatorOptions_t *Create_Options( void );
void Print_Options( TranslatorOptions_t *to );
void Free_Options( TranslatorOptions_t *to );

char *Get_EVENTS_Name( TranslatorOptions_t *to );
char *New_File_Name( TranslatorOptions_t *to, const char* suffix );

#endif /* _PARSE_PARAMETERS_H_ */
