/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file    records.h
 *
 *  @brief   Event and Communication records definition.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/

#ifndef _RECORDS_H_
#define _RECORDS_H_

#include <inttypes.h>
#include "types.h"

#define MAX_THREAD 65536

#define FORMAT_STATE_RECORD "1:%d:%d:%d:%d:%lld:%lld:d\n"
#define FORMAT_EVENT_RECORD "2:%d:%d:%d:%d:%lld:%d:%lld"
#define FORMAT_EVENT_RECORD_REDUCED "2:%d:%d:%d:%d:%lld"
#define FORMAT_EVENT_RECORD_COMPACT ":%d:%lld"
#define FORMAT_COMMS_RECORD "3:%d:%d:%d:%d:%lld:%lld:%d:%d:%d:%d:%lld:%lld:%d:%d\n"
#define FORMAT_COMMS_RECORD_REDUCED "3:%d:%d:%d:%d:%lld"

#define PRV_RECORD_TYPES 2

#define MPI_SEND  0
#define MPI_RECV  1
#define MPI_ISEND 2
#define MPI_IRECV 3
#define MPI_WAIT  4
#define MPI_UNKNOWN 5

typedef struct
{
  uint32_t cpu;
  uint32_t ptask;
  uint32_t task;
  uint32_t thread;
  uint64_t time;
  uint32_t type;
  uint32_t value;
} EventRecord_t;

typedef struct
{
  uint32_t snd_cpu;
  uint32_t snd_ptask;
  uint32_t snd_task;
  uint32_t snd_thread;
  uint64_t snd_time; /* No logical-physical time information through OTF. */

  uint32_t rcv_cpu;
  uint32_t rcv_ptask;
  uint32_t rcv_task;
  uint32_t rcv_thread;
  uint64_t rcv_time;

  uint32_t size;
  uint32_t tag;
  uint32_t type;
  bool_t  matched;
} CommRecord_t;


EventRecord_t *CreateEventRecord( uint32_t cpu,
                                    uint32_t ptask,
                                    uint32_t task,
                                    uint32_t thread,
                                    uint64_t time,
                                    uint32_t type,
                                    uint32_t value );
void CopyEventRecord( EventRecord_t *src, EventRecord_t *dst );
EventRecord_t *CloneEventRecord( EventRecord_t *src );
void FreeEventRecord( EventRecord_t *er );

/* CommRecord_t functions */
CommRecord_t   *CreateCommRecord( uint32_t snd_cpu,
                                    uint32_t snd_ptask,
                                    uint32_t snd_task,
                                    uint32_t snd_thread,
                                    uint64_t snd_time,
                                    uint32_t rcv_cpu,
                                    uint32_t rcv_ptask,
                                    uint32_t rcv_task,
                                    uint32_t rcv_thread,
                                    uint64_t rcv_time,
                                    uint32_t size,
                                    uint32_t tag,
                                    uint32_t type );
void CopyCommRecord( CommRecord_t *src, CommRecord_t *dst );
bool_t AreCommsEqual( CommRecord_t *src, CommRecord_t *dst );
void FreeCommRecord( CommRecord_t *er );

#endif /* _RECORDS_H_ */
