/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file   statistics.h
 *
 *  @brief  Structures to save statistics and show final briefing.
 *
 *  @author Pedro A. Gonzalez - CEPBA-tools
*/

#ifndef _STATISTICS_H_
#define _STATISTICS_H_

#include "records.h"
#include "parse_parameters.h"

/* Definition records */
#define OTF_DEFVERSION_REC          0
#define OTF_DEFCREATOR_REC          1
#define OTF_DEFCOMMENT_REC          2
#define OTF_DEFTIMERRESOLUTION_REC  3
#define OTF_DEFPROCESSGROUP_REC     4
#define OTF_DEFPROCESS_REC          5
#define OTF_DEFFUNCTIONGROUP_REC    6
#define OTF_DEFFUNCTION_REC         7
#define OTF_DEFCOUNTERGROUP_REC     8
#define OTF_DEFCOUNTER_REC          9
#define OTF_DEFCOLLOP_REC          10
#define OTF_DEFSCL_REC             11 
#define OTF_DEFSCLFILE_REC         12

/* Event records*/
#define OTF_EVENTCOMMENT_REC       13
#define OTF_ENTER_REC              14
#define OTF_LEAVE_REC              15
#define OTF_SEND_REC               16
#define OTF_RECEIVE_REC            17
#define OTF_COUNTER_REC            18
#define OTF_COLLOP_REC             19
#define OTF_BEGINPROCESS_REC       20
#define OTF_ENDPROCESS_REC         21

/* Summary records*/
#define OTF_SUMMARYCONTENT_REC        22
#define OTF_FUNCTIONSUMMARY_REC       23
#define OTF_FUNCTIONGROUPSUMMARY_REC  24
#define OTF_MESSAGESUMMARY_REC        25

/* Snapshot records*/
#define OTF_SNAPSHOTCOMMENT_REC    26
#define OTF_ENTERSNAPSHOT_REC      27
#define OTF_SENDSNAPSHOT_REC       28

/* Number of records */
#define OTF_RECORD_TYPES           29

#define OTF_INI_DEF_REC     OTF_DEFVERSION_REC
#define OTF_END_DEF_REC     OTF_DEFSCLFILE_REC

#define OTF_INI_EVT_REC     OTF_EVENTCOMMENT_REC
#define OTF_END_EVT_REC     OTF_ENDPROCESS_REC

#define OTF_INI_SUM_REC     OTF_SUMMARYCONTENT_REC
#define OTF_END_SUM_REC     OTF_MESSAGESUMMARY_REC

#define OTF_INI_SNA_REC     OTF_SNAPSHOTCOMMENT_REC
#define OTF_END_SNA_REC     OTF_SENDSNAPSHOT_REC

typedef struct
{
  char *name;
} Record_Name_t;


#define PRV_EVENTS         0
#define PRV_COMMUNICATIONS 1

#define QSENDS      0
#define QRECEIVES   1
#define QEVENTS     2
#define QUEUE_TYPES 2 /* Events queue not used */


typedef struct
{
  unsigned int      queued;
  unsigned int      dequeued;
  long unsigned int total_sum_queue;
} QueueStatistics_t;


typedef struct
{
  unsigned int otf_num_records_read;
  unsigned int otf_records[ OTF_RECORD_TYPES ];
  unsigned int otf_unmatched_sends;
  unsigned int otf_unmatched_receives;
  unsigned int otf_matched_sends;
  unsigned int otf_matched_receives;

  unsigned int prv_num_records_wrote;
  unsigned int prv_records[ PRV_RECORD_TYPES ];
  unsigned int prv_negative_comms;

  QueueStatistics_t queue[ QUEUE_TYPES ];
} Statistics_t;

void Init_Statistics( Statistics_t *st );

void Print_Statistics( Statistics_t *stats,
                       TranslatorOptions_t *param,
											 int argc,
											 char *argv[] );

#endif /* _STATISTICS_H_ */
