/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file    reader.c
 *
 *  @brief   OTF handler settings and translate function.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/

#include "OTF_Reader.h"
#include "reader.h"
#include "read_handlers.h"
#include "otf_symbolic_info.h"
#include "parse_parameters.h"
#include "tokens.h"

extern TranslatorOptions_t *parameters;

static int HandlerArray_setReadHandler( OTF_HandlerArray* handlers,
                                         OTF_Reader* rd,
                                         FILE *prv_fd )
{
	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefinitionComment,
		OTF_DEFINITIONCOMMENT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFINITIONCOMMENT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefTimerResolution,
		OTF_DEFTIMERRESOLUTION_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFTIMERRESOLUTION_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefProcess,
		OTF_DEFPROCESS_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFPROCESS_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefProcessGroup,
		OTF_DEFPROCESSGROUP_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFPROCESSGROUP_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefFunction,
		OTF_DEFFUNCTION_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFFUNCTION_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefFunctionGroup,
		OTF_DEFFUNCTIONGROUP_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFFUNCTIONGROUP_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefCollectiveOperation,
		OTF_DEFCOLLOP_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFCOLLOP_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefCounter,
		OTF_DEFCOUNTER_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFCOUNTER_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefCounterGroup,
		OTF_DEFCOUNTERGROUP_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFCOUNTERGROUP_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefScl,
		OTF_DEFSCL_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFSCL_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefSclFile,
		OTF_DEFSCLFILE_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFSCLFILE_RECORD );


	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_DefCreator,
		OTF_DEFCREATOR_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_DEFCREATOR_RECORD );

  if ( !parameters->option[ OPTION_I2 ] )
  {
	  OTF_HandlerArray_setHandler( handlers,
		  (OTF_FunctionPointer*) ReadHandler_Enter,
	  	OTF_ENTER_RECORD );
	  OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		  OTF_ENTER_RECORD );

	  OTF_HandlerArray_setHandler( handlers,
		  (OTF_FunctionPointer*) ReadHandler_Leave,
		  OTF_LEAVE_RECORD );
	  OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		  OTF_LEAVE_RECORD );
  }

  if ( !parameters->option[ OPTION_I3 ] )
  {
  	OTF_HandlerArray_setHandler( handlers,
	  	(OTF_FunctionPointer*) ReadHandler_SendMsg,
  		OTF_SEND_RECORD );
  	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
	  	OTF_SEND_RECORD );

	  OTF_HandlerArray_setHandler( handlers,
  		(OTF_FunctionPointer*) ReadHandler_RecvMsg,
	  	OTF_RECEIVE_RECORD );
	  OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
  		OTF_RECEIVE_RECORD );
  }

  if ( !parameters->option[ OPTION_I2 ] )
  {
  	OTF_HandlerArray_setHandler( handlers,
	  	(OTF_FunctionPointer*) ReadHandler_Counter,
		  OTF_COUNTER_RECORD );
	  OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		  OTF_COUNTER_RECORD );
  }

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_CollectiveOperation,
		OTF_COLLOP_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_COLLOP_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_EventComment,
		OTF_EVENTCOMMENT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_EVENTCOMMENT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_BeginProcess,
		OTF_BEGINPROCESS_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_BEGINPROCESS_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_EndProcess,
		OTF_ENDPROCESS_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_ENDPROCESS_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_SnapshotComment,
		OTF_SNAPSHOTCOMMENT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_SNAPSHOTCOMMENT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_EnterSnapshot,
		OTF_ENTERSNAPSHOT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_ENTERSNAPSHOT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_SendSnapshot,
		OTF_SENDSNAPSHOT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_SENDSNAPSHOT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_SummaryComment,
		OTF_SUMMARYCOMMENT_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_SUMMARYCOMMENT_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_FunctionSummary,
		OTF_FUNCTIONSUMMARY_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_FUNCTIONSUMMARY_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_FunctionGroupSummary,
		OTF_FUNCTIONGROUPSUMMARY_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_FUNCTIONGROUPSUMMARY_RECORD );

	OTF_HandlerArray_setHandler( handlers,
		(OTF_FunctionPointer*) ReadHandler_MessageSummary,
		OTF_MESSAGESUMMARY_RECORD );
	OTF_HandlerArray_setFirstHandlerArg( handlers, rd,
		OTF_MESSAGESUMMARY_RECORD );

	return 1;
}

int Translate( char *trace_name,
               char *rstream_name,
               FILE *prv_fd,
               FILE *pcf_fd )
{
  OTF_FileManager *fm;
  OTF_Reader *rd;
  OTF_HandlerArray *handler;
  int err = 0;
  int count_df = 0, count_st = 0, count_ev = 0;
  OTF_RStream *stream;

  /* max num of files allowed to be open simult. */
  fm = OTF_FileManager_open( 1 );
  rd = OTF_Reader_open( trace_name, fm );
  handler = OTF_HandlerArray_open();
  if (handler == NULL)
  {
    printf( "Error opening HandlerArray\n" );
    return -1;
  }

  /* CREATE AUX STRUCTURES */
  sym_otf_info = Create_Otf_Symbolic_Info();

  HandlerArray_setReadHandler( handler, rd, prv_fd );

  /* read global definitions */
  count_df = OTF_Reader_readDefinitions( rd, handler );
  count_ev = OTF_Reader_readEvents( rd, handler );
  count_st = OTF_Reader_readStatistics( rd, handler );

  /* read local  definitions */
  stream = OTF_Reader_getStream( rd, 1 );
  if ( stream != NULL )
  {

    stream = OTF_RStream_open( rstream_name, 1, fm );
    count_df = OTF_RStream_readDefinitions( stream, handler );
  }

  /* close files */
  if (stream != NULL)
    OTF_RStream_close( stream );

  err = OTF_HandlerArray_close( handler );
  err = OTF_Reader_close( rd );
  OTF_FileManager_close( fm );

  return err;
}
