/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                                   otf2prv                                 *
 *                    Trace translator from otf to prv format                *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/**
 *  @file  records.c
 *
 *  @brief  Event and Communication records ops.
 *
 *  @author  Pedro A. Gonzalez - CEPBA-tools
*/

#include <stdlib.h>
#include <stdio.h>

#include "records.h"


EventRecord_t *CreateEventRecord( uint32_t cpu,
                                    uint32_t ptask,
                                    uint32_t task,
                                    uint32_t thread,
                                    uint64_t time,
                                    uint32_t type,
                                    uint32_t value )
{
  EventRecord_t *er;

  er = ( EventRecord_t * )malloc( sizeof( EventRecord_t ));
  if ( er == NULL )
  {
    printf( "Out of Memory in %s:%i.\n", __FUNCTION__, __LINE__ );
    exit( 1 );
  }

  er->cpu    = cpu;
  er->ptask  = ptask;
  er->task   = task;
  er->thread = thread;
  er->time   = time;
  er->type   = type;
  er->value  = value;

  return er;
}


void CopyEventRecord( EventRecord_t *src, EventRecord_t *dst )
{
  if (( src != NULL ) && ( dst != NULL ))
  {
    dst->cpu    = src->cpu;
    dst->ptask  = src->ptask;
    dst->task   = src->task;
    dst->thread = src->thread;
    dst->time   = src->time;
    dst->type   = src->type;
    dst->value  = src->value;
  }
}


EventRecord_t *CloneEventRecord( EventRecord_t *src )
{
  EventRecord_t *dst;

  dst = ( EventRecord_t * )malloc( sizeof( EventRecord_t ));
  if ( dst == NULL )
  {
    printf( "Out of Memory in %s:%i.\n", __FUNCTION__, __LINE__ );
    exit( 1 );
  }

  CopyEventRecord( src, dst );

  return dst;
}


void FreeEventRecord( EventRecord_t *er )
{
  free( er );
	er = NULL;
}


CommRecord_t *CreateCommRecord( uint32_t snd_cpu,
                                  uint32_t snd_ptask,
                                  uint32_t snd_task,
                                  uint32_t snd_thread,
                                  uint64_t snd_time,
                                  uint32_t rcv_cpu,
                                  uint32_t rcv_ptask,
                                  uint32_t rcv_task,
                                  uint32_t rcv_thread,
                                  uint64_t rcv_time,
                                  uint32_t size,
                                  uint32_t tag,
                                  uint32_t type )
{
  CommRecord_t *cr;

  cr = ( CommRecord_t * )malloc( sizeof( CommRecord_t ));
  if ( cr == NULL )
  {
    printf( "Out of Memory in %s:%i.\n", __FUNCTION__, __LINE__ );
    exit( 1 );
  }

  cr->snd_cpu    = snd_cpu;
  cr->snd_ptask  = snd_ptask;
  cr->snd_task   = snd_task;
  cr->snd_thread = snd_thread;
  cr->snd_time   = snd_time;

  cr->rcv_cpu    = rcv_cpu;
  cr->rcv_ptask  = rcv_ptask;
  cr->rcv_task   = rcv_task;
  cr->rcv_thread = rcv_thread;
  cr->rcv_time   = rcv_time;

  cr->size    = size;
  cr->tag     = tag;
  cr->type    = type;
  cr->matched = FALSE;

  return cr;
}

void CopyCommRecord( CommRecord_t *src, CommRecord_t *dst )
{
  if (( src != NULL ) && ( dst != NULL ))
  {
    src->snd_cpu    = dst->snd_cpu;
    src->snd_ptask  = dst->snd_ptask;
    src->snd_task   = dst->snd_task;
    src->snd_thread = dst->snd_thread;
    src->snd_time   = dst->snd_time;

    src->rcv_cpu    = dst->rcv_cpu;
    src->rcv_ptask  = dst->rcv_ptask;
    src->rcv_task   = dst->rcv_task;
    src->rcv_thread = dst->rcv_thread;
    src->rcv_time   = dst->rcv_time;

    src->size       = dst->size;
    src->tag        = dst->tag;
    src->type       = dst->type;
    src->matched    = dst->matched;
  }
}


bool_t AreCommsEqual( CommRecord_t *src, CommRecord_t *dst )
{
  bool_t equals = FALSE;

  if (( src != NULL ) && ( dst != NULL ))
  {
    equals =
    ((src->snd_cpu    == dst->snd_cpu) &&
     (src->snd_ptask  == dst->snd_ptask) &&
     (src->snd_task   == dst->snd_task) &&
     (src->snd_thread == dst->snd_thread) &&
     (src->rcv_cpu    == dst->rcv_cpu) &&
     (src->rcv_ptask  == dst->rcv_ptask) &&
     (src->rcv_task   == dst->rcv_task) &&
     (src->rcv_thread == dst->rcv_thread) &&
     (src->size       == dst->size) &&
     (src->tag        == dst->tag));
  }

  return equals;
}


void FreeCommRecord( CommRecord_t *cr )
{
  free( cr );
	cr = NULL;
}
